package Active;

/**
	One-Slot Buffer implements safe transfer of information
	between Producer and Consumer.
	This is is a generic component that can be used to coordinate
	communication between two threads.  Note that only Objects
	can be stored, not primitive types.
*/

class OneSlotBuffer {
	private Object slotVal; // initially null

	public synchronized void put(Object val) {
		while (slotVal != null) {
			// System.out.println("OneSlotBuffer full -- waiting");
			try {
				wait();
			}  catch (InterruptedException e) {
				System.out.println("OneSlotBuffer interrupted!");
			}
		}
		slotVal = val;
		notify();
		return;
	}

	public synchronized Object get() {
		Object rval;
		while (slotVal == null) {
			// System.out.println("OneSlotBuffer empty -- waiting");
			try {
				wait();
			}  catch (InterruptedException e) {
				System.out.println("OneSlotBuffer interrupted!");
			}
		}
		rval = slotVal;
		slotVal = null;
		notify();
		return rval;
	}
}
