/*
  File: ArcPanel.java
*/

package support;

import java.awt.*;

public class ArcPanel extends Panel
{

    Font font_ = new Font("Times",Font.ITALIC+Font.BOLD,24);

    Label title_;

    ArcCanvas arcCanvas_;

    static final int STEP = 6;

    Color arcColor_ = Color.blue;

    public ArcPanel( String title )
    {
	this( new BorderLayout(), title, Color.cyan );
    }

    public ArcPanel( String title, Color bgc )
    {
	this( new BorderLayout(), title, bgc );
    }

    public ArcPanel( LayoutManager layout, String title )
    {
	this( layout, title, Color.cyan );
    }

    public ArcPanel( LayoutManager layout, String title, Color bgc )
    {
	super( layout );
	setBackground( bgc );

	title_ = new Label( title, Label.CENTER );
	title_.setFont( font_ );
	add( title_, BorderLayout.NORTH );

	Panel p = new Panel( new BorderLayout() );
	p.add( new Label( "90", Label.CENTER ), BorderLayout.NORTH );
	arcCanvas_ = new ArcCanvas( 0, 175, arcColor_ );
	p.add( arcCanvas_, BorderLayout.CENTER );
	p.add( new Label( "270", Label.CENTER ), BorderLayout.SOUTH );
	p.add( new Label( "0", Label.LEFT ), BorderLayout.EAST );
	p.add( new Label( "180", Label.RIGHT ), BorderLayout.WEST );
	add( p, BorderLayout.CENTER );
    }

    public void doStep()
    {
	arcCanvas_.setAngle( arcCanvas_.getAngle() + STEP );
    }

    public void rotate()
    {
	doStep();
    }

    public void setZero()
    {
	arcCanvas_.start_ = 0;
	arcCanvas_.setAngle( 0 );
    }

    public void setArcColor( Color c )
    {
	arcColor_ = c;
	arcCanvas_.setColor( c );
    }

    public Color getColor()
    {
	return arcColor_;
    }

}

class ArcCanvas extends Canvas
{

    int start_;

    int angle_;

    Color arcColor_;

    boolean repaint_;

    public ArcCanvas( int start, int angle, Color arcColor )
    {
	super();
	start_ = start % 360;
	angle_ = angle % 360;
	arcColor_ = arcColor;
	setSize( 100, 100 );
        repaint_ = true;
    }

    public void setStart( int start )
    {
	start_ = start % 360;
	repaint();
    }

    public int getStart()
    {
	return start_;
    }

    public void setAngle( int angle )
    {
        if ( angle_ > 360 )
            {
                angle_ = angle % 360;
                repaint_ = true;
            }
        else
          angle_ = angle;
	repaint();
    }

    public int getAngle()
    {
	return angle_;
    }

    public void setColor( Color c )
    {
	arcColor_ = c;
	repaint();
    }

    public void update( Graphics g )
    {
        if ( repaint_ )
            {
                repaint_ = false;
                super.update( g ); // clear circle
            }
        paint( g );
    }

    public void paint( Graphics g )
    {
	g.setColor( arcColor_ );
	g.fillArc( 0, 0, getSize().width, getSize().height,
		   start_, angle_ );
    }

}

