/*
  File: ArcPanel.java
*/

package support;

import java.awt.*;
import java.awt.event.*;

public class ArcPanel extends Panel
{

    Font font_ = new Font("Times",Font.ITALIC+Font.BOLD,24);

    Label title_;

    ArcCanvas arcCanvas_;

    static final int STEP = 6;

    Color arcColor_ = Color.blue;

    LineAdjustment l_;

    public ArcPanel( String title )
    {
	this( title, null, Color.cyan );
    }

    public ArcPanel( String title, CriticalSection controller )
    {
	this( title, controller, Color.cyan );
    }

    public ArcPanel( String title, Color bgc )
    {
	this( title, null, bgc );
    }

    public ArcPanel( String title, CriticalSection controller, Color bgc )
    {
	super( new BorderLayout() );

	setBackground( bgc );

	title_ = new Label( title, Label.CENTER );
	title_.setFont( font_ );
	add( title_, BorderLayout.NORTH );

	Panel p = new Panel( new BorderLayout() );
	p.add( new Label( "90", Label.CENTER ), BorderLayout.NORTH );
	arcCanvas_ = new ArcCanvas( 0, 0, controller, arcColor_ );
	l_ = new LineAdjustment( arcCanvas_ );
	p.add( arcCanvas_, BorderLayout.CENTER );
	p.add( new Label( "270", Label.CENTER ), BorderLayout.SOUTH );
	p.add( new Label( "0", Label.LEFT ), BorderLayout.EAST );
	p.add( new Label( "180", Label.RIGHT ), BorderLayout.WEST );
	add( p, BorderLayout.CENTER );
    }

    public void doStep() throws InterruptedException
    {
	arcCanvas_.setAngle( arcCanvas_.getAngle() + STEP );
    }

    public void rotate() throws InterruptedException
    {
	doStep();
    }

    public void setZero() throws InterruptedException
    {
	arcCanvas_.start_ = 0;
	arcCanvas_.setAngle( 0 );
    }

    public void enableChangeOfCriticalSection()
    {
	if ( arcCanvas_.controller_ != null )
	    {
		arcCanvas_.addMouseListener( l_ );
		arcCanvas_.addMouseMotionListener( l_ );
		arcCanvas_.enableChangeOfCriticalSection();
	    }
    }

    public void disableChangeOfCriticalSection() throws InterruptedException
    {
	if ( arcCanvas_.controller_ != null )
	    {
		arcCanvas_.disableChangeOfCriticalSection();
		arcCanvas_.removeMouseListener( l_ );
		arcCanvas_.removeMouseMotionListener( l_ );
	    }
    }

    public void setArcColor( Color c )
    {
	arcColor_ = c;
	arcCanvas_.setColor( c );
    }

    public Color getArcColor()
    {
	return arcColor_;
    }

    public int getAngle()
    {
	return arcCanvas_.getAngle();
    }

}

class LineAdjustment implements MouseListener,
				MouseMotionListener
{

    ArcCanvas owner_;

    boolean start_;

    boolean stop_;

    
    public LineAdjustment( ArcCanvas owner )
    {
	owner_ = owner;
    }

    // MouseListerner
   
    public void mouseClicked( MouseEvent e )
    {
    }

    public void mouseEntered( MouseEvent e )
    {
    }

    public void mouseExited( MouseEvent e )
    {
    }

    public void mousePressed( MouseEvent e )
    {
	start_ = false;
	stop_ = false;

	// Is pointer near a line

	int selangle = owner_.getSelectedAngle( e.getPoint() );
	if ( owner_.isStartLineSelected( selangle ) &&
	     (owner_.looking_ == false) )
	    {
		start_ = true;
		owner_.setCursor( 
		      Cursor.getPredefinedCursor( Cursor.HAND_CURSOR ) );
	    }
	else
	    {
		if ( owner_.isStopLineSelected( selangle ) )
		    {
			stop_ = true;
			owner_.setCursor( 
				    Cursor.getPredefinedCursor( 
						    Cursor.HAND_CURSOR ) );
		    }
	    }
    }

    public void mouseReleased( MouseEvent e )
    {
	int newangle = owner_.getSelectedAngle( e.getPoint() );
	newangle = newangle + ArcPanel.STEP - (newangle % ArcPanel.STEP);
	if ( start_ )
	    owner_.setStartCriticalSection( newangle );
	if ( stop_ )
	    owner_.setStopCriticalSection( newangle );
	owner_.setCursor( Cursor.getDefaultCursor() );
    }

    // MouseMotionListener

    public void mouseDragged( MouseEvent e )
    {
	int newangle = owner_.getSelectedAngle( e.getPoint() );
	if ( start_ )
	    owner_.setStartCriticalSection( newangle );
	if ( stop_ )
	    owner_.setStopCriticalSection( newangle );
    }

    public void mouseMoved( MouseEvent e )
    {
    }

}

class ArcCanvas extends Canvas
{

    int start_;

    int angle_;

    Color arcColor_;

    Color criticalSectionColor_ = Color.cyan;

    int startCriticalSection_ = 90;

    int stopCriticalSection_ = 270;

    final int EPSILON = 3;

    boolean canChangeCriticalSection_ = true;

    boolean repaint_ = false;

    CriticalSection controller_;

    boolean inCriticalSection_ = false;

    boolean mustLock_ = false;

    boolean mustRelease_ = false;

    boolean looking_ = false;

    public ArcCanvas( int start, int angle, 
		      CriticalSection controller,
		      Color arcColor )
    {
	super();
	start_ = start % 360;
	angle_ = angle % 360;
	arcColor_ = arcColor;
	setSize( 100, 100 );
	controller_ = controller;
    }

    public void enableChangeOfCriticalSection()
    {
	canChangeCriticalSection_ = true;
	mustLock_ = false;
	mustRelease_ = false;
    }

    public void disableChangeOfCriticalSection() throws InterruptedException
    {
	canChangeCriticalSection_ = false;
    }

    private void enterCriticalSection() throws InterruptedException
    {
	if ( controller_ != null )
	    {
		controller_.enterCriticalSection();
		inCriticalSection_ = true;
	    }
    }

    private void leaveCriticalSection()
    {
	if ( controller_ != null )
	    {
		controller_.leaveCriticalSection();
		inCriticalSection_ = false;
	    }
    }

    public void setStartCriticalSection( int angle )
    {
	mustLock_ = false;
	mustRelease_ = false;
	if ( inCriticalSection_ )
	    {
		if ( (angle >= angle_) &&
		     (angle < stopCriticalSection_) )
		    {
			mustRelease_ = true;
		    }
		startCriticalSection_ = angle;
		repaint();
	    }
	else
	    {
		if ( !looking_ )
		    {
			if ( (angle < angle_) ||
			     (stopCriticalSection_ <= angle) )
			    {
				mustLock_ = true;
			    }
			startCriticalSection_ = angle;
			repaint();
		    }
	    }
    }

    public void setStopCriticalSection( int angle )
    {
	mustLock_ = false;
	mustRelease_ = false;
	if ( inCriticalSection_ )
	    {
		if ( (angle <= angle_) &&
		     (stopCriticalSection_ > startCriticalSection_) )
		    {
			mustRelease_ = true;
		    }
		stopCriticalSection_ = angle;
		repaint();
	    }
	else
	    {
		if ( (angle >= angle_) &&
		     ((angle_ > startCriticalSection_) ||
		      (angle < startCriticalSection_)) ) 
		    {
			mustLock_ = true;
		    }
		stopCriticalSection_ = angle;
		repaint();
	    }
    }

    public int getSelectedAngle( Point point )
    {
	int xDelta = point.x - (getSize().width / 2);
	int yDelta = ((getSize().height / 2) - point.y) ;
	double selAlpha = Math.atan2( (double)yDelta, (double)xDelta );
	return ((int)(selAlpha * 180.0 / Math.PI) + 360 ) % 360;
    }

    public boolean isStartLineSelected( int selangle )
    {
	return ((startCriticalSection_ - EPSILON) <= selangle) &&
	       ((startCriticalSection_ + EPSILON) >= selangle);
    }

    public boolean isStopLineSelected( int selangle )
    {
	return ((stopCriticalSection_ - EPSILON) <= selangle) &&
	       ((stopCriticalSection_ + EPSILON) >= selangle);
    }

    public void setStart( int start )
    {
	start_ = start % 360;
	repaint();
    }

    public int getStart()
    {
	return start_;
    }

    public void setAngle( int angle ) throws InterruptedException
    {
	// critical section has been changed manually
	if ( controller_ != null )
	    {
		if ( mustLock_ )
		    {
			enterCriticalSection();
			mustLock_ = false;
		    }
		if ( mustRelease_ )
		    {
			leaveCriticalSection();
			mustRelease_ = false;
		    }
	    }

	if ( angle > 360 )
	    {
		angle_ = angle % 360;
		repaint_ = true;
	    }
	else
	    angle_ = angle;
	repaint();

	// critical section handling
	if ( controller_ != null )
	    {
		if ( !inCriticalSection_ &&
		     (angle_ == startCriticalSection_) )
		    {
			looking_ = true;
			enterCriticalSection();
			looking_ = false;
		    }
		if ( inCriticalSection_ &&
		     (angle_ == stopCriticalSection_) )
		    {
			leaveCriticalSection();
		    }
	    }
    }

    public int getAngle()
    {
	return angle_;
    }

    public void setColor( Color c )
    {
	arcColor_ = c;
	repaint();
    }

    public void update( Graphics g )
    {
	paint( g );
    }

    public void paint( Graphics g )
    {
	Dimension d = getSize();

	// clear background if necessary
	if ( repaint_ || canChangeCriticalSection_ )
	    {
		super.paint( g );
		repaint_ = false;
	    }

	if ( controller_ != null )
	    {
		// draw critical section
		g.setColor( criticalSectionColor_ );
		if ( startCriticalSection_ < stopCriticalSection_ )
		    g.fillArc( 0, 0, d.width, d.height, 
			       startCriticalSection_, 
			       stopCriticalSection_ - startCriticalSection_ );
		else
		    g.fillArc( 0, 0, d.width, d.height, 
			       startCriticalSection_, 
			       stopCriticalSection_ + 
			                     (360 - startCriticalSection_) );

		// draw actual arc
		g.setColor( arcColor_ );
		g.fillArc( 5, 5, d.width - 10, d.height - 10, start_, angle_ );
		
		// draw limits
		g.setColor( Color.black );
		
		// start
		double alpha = 2 * Math.PI * startCriticalSection_ / 360.0;
		int r = getSize().width / 2;
		double x1 = Math.cos( alpha ) * r;
		double y1 = Math.sin( alpha ) * r;
		int zeroX = d.width / 2;
		int zeroY = d.height / 2;
		
		g.drawLine( zeroX, zeroY, zeroX + (int)x1, zeroY - (int)y1 );
		
		// stop
		alpha = 2 * Math.PI * stopCriticalSection_ / 360.0;
		x1 = Math.cos( alpha ) * r;
		y1 = Math.sin( alpha ) * r;
		
		g.drawLine( zeroX, zeroY, zeroX + (int)x1, zeroY - (int)y1 );
	    }
	else
	    {
		// draw actual arc
		g.setColor( arcColor_ );
		g.fillArc( 0, 0, d.width, d.height, start_, angle_ );
	    }
    }

}

